/*
 * spec: empty
 */
/*
 * spec.vue : vue
 * spec.navigation: navigation
 */
var SCENE;
var Scene=(function () {
    return {
        instance: function(spec) {
            spec.navigation=spec.navigation || false;

            var objets=[],
                objets_alpha=[],
                lumieres=[];

            var drawObjet=function(objet) {
                if (!objet.loaded) return false;
                objet.draw();
                return true;
            }

            var drawObjetDistance=function(objet) {
                if (!objet.loaded) return false;
                objet.drawDistance();
                return true;
            }

            var drawObjetPhysics=function(objet) {                
                if (objet.drawPhysics) objet.drawPhysics(objet, dt);
            }

            GL.enable(GL.DEPTH_TEST);
            GL.depthFunc(GL.LEQUAL);            
            GL.clearDepth(1.0);
            GL.blendFunc(GL.ONE, GL.ONE_MINUS_SRC_ALPHA);
            SHADERS.set_samplers();
            
            var old_timestamp=0, dt=0;
            var physicsW=SETTINGS.physics.cubemap_size,
            zMin=SETTINGS.physics.z_min,
            zMax=SETTINGS.physics.z_max;
            var proj_matrix_cubeMap=lib_matrix_projection.cubeMatrices(zMin, zMax),
            debugDistance=false;

            //precompute theta, phi

            /*   | Y
             *   |
             *   |
             *   |_________ X
             *  /
             *Z/
             */
            var n=physicsW*physicsW;
            var theta_buffers={
                front: new Float32Array(n),
                back :  new Float32Array(n),
                left:  new Float32Array(n),
                right:  new Float32Array(n),
                top:  new Float32Array(n),
                bottom:  new Float32Array(n)
            },

            phi_buffers={
                front: new Float32Array(n),
                back :  new Float32Array(n),
                left:  new Float32Array(n),
                right:  new Float32Array(n),
                top:  new Float32Array(n),
                bottom:  new Float32Array(n)
            }

            var preComputeThetaPhi=function(){
                var x,y,i, max=0, theta, phi, ang_x, ang_y, ang_x_pole, ang_x_pole2, ang_y_pole,xx,yy;
                for (x=0; x<physicsW; x++) {
                    ang_x=(x*(Math.PI/2)/physicsW)-Math.PI/4;
                    
                    xx=2*((x/physicsW)-0.5)*Math.sqrt(2)/2;

                    for (y=0; y<physicsW; y++){
                        yy=2*((y/physicsW)-0.5)*Math.sqrt(2)/2;
                        //ang_y=(y*(Math.PI/2)/physicsW)-Math.PI/4;
                        ang_y=Math.atan2(yy, Math.sqrt(xx*xx+0.5));

                        ang_x_pole=Math.atan2(x-physicsW/2, y-physicsW/2);
                        ang_x_pole2=Math.atan(Math.PI/2+(x-physicsW/2)/(y-physicsW/2));
                        ang_y_pole=Math.atan(Math.sqrt(2*(xx*xx+yy*yy)));


                        theta=-ang_x,
                        phi=ang_y;

                        i=x+y*physicsW;

                        //-z axis
                        theta_buffers.front[i]=theta+Math.PI,
                        phi_buffers.front[i]=phi+Math.PI/2;

                        //z axis
                        theta_buffers.back[i]=theta,
                        phi_buffers.back[i]=phi+Math.PI/2;

                        //x axis
                        theta_buffers.right[i]=theta+Math.PI/2,
                        phi_buffers.right[i]=phi+Math.PI/2;

                        //-x axis
                        theta_buffers.left[i]=theta-Math.PI/2+2*Math.PI,
                        phi_buffers.left[i]=phi+Math.PI/2;

                        //y axis //bottom
                        theta_buffers.top[i]=ang_x_pole,
                        phi_buffers.top[i]=Math.PI-ang_y_pole;

                        //-y axis //top
                        theta_buffers.bottom[i]=-ang_x_pole+Math.PI,
                        phi_buffers.bottom[i]=ang_y_pole;
                    }
                }
            }

            var debugThetaPhi=function() {
                var debugCanvas=document.createElement("canvas");
                debugCanvas.width=physicsW*4, debugCanvas.height=physicsW*3;
                debugCanvas.style.position="absolute",
                debugCanvas.style.top="0px",
                debugCanvas.style.left="0px";

                var ctx=debugCanvas.getContext("2d");
                document.body.appendChild(debugCanvas);

                var debugThetaPhiBuffer=function(thetas, phis, offset_x, offset_y){
                    var image_data=ctx.createImageData(physicsW,physicsW);
                    var x,y,i, theta, phi;
                    for (x=0; x<physicsW; x++) {
                        for (y=0; y<physicsW; y++){
                            i=(x+y*physicsW);
                            theta=thetas[i], phi=phis[i];
                            image_data.data[i*4]=255*(phi/Math.PI), //red
                            image_data.data[i*4+1]=255*((theta+2*Math.PI)/(4*Math.PI)), //green
                            image_data.data[i*4+2]=0, //blue
                            image_data.data[i*4+3]=255; //alpha
                        }
                    }
                    ctx.putImageData(image_data, offset_x*physicsW, offset_y*physicsW);
                }

                debugThetaPhiBuffer(theta_buffers.front, phi_buffers.front, 1,1),
                debugThetaPhiBuffer(theta_buffers.bottom, phi_buffers.bottom, 1,0),

                debugThetaPhiBuffer(theta_buffers.top, phi_buffers.top, 1,2),
                debugThetaPhiBuffer(theta_buffers.left, phi_buffers.left, 0,1),

                debugThetaPhiBuffer(theta_buffers.right, phi_buffers.right, 2,1),
                debugThetaPhiBuffer(theta_buffers.back, phi_buffers.back, 3,1);

                //ctx.fillStyle="red";
                //ctx.fillRect(0,0,physicsW, physicsW);
            }
            preComputeThetaPhi();
            //debugThetaPhi();           



            var n=4*physicsW*physicsW;
            var cubemaps_buffers = {
                front: new Uint8Array(n),
                back :  new Uint8Array(n),
                left:  new Uint8Array(n),
                right:  new Uint8Array(n),
                top:  new Uint8Array(n),
                bottom:  new Uint8Array(n)
            }

            var get_max=function(side) {
                var theta_buf=theta_buffers[side],
                    phi_buf=phi_buffers[side],
                    buf=cubemaps_buffers[side];

                var x,y,z,i, max=0, thetaMax=0, phiMax=0, r=0;
                for (x=0; x<physicsW; x++) {
                    for (y=0; y<physicsW; y++){
                        i=(x+y*physicsW);
                        r=buf[i*4];
                        if (r>max){                            
                            max=r;
                            thetaMax=theta_buf[i],
                            phiMax=phi_buf[i];
                        }
                    }
                }
                max/=255;

                z=max*Math.cos(thetaMax)*Math.sin(phiMax);
                x=max*Math.sin(thetaMax)*Math.sin(phiMax);
                y=max*Math.cos(phiMax);

                return {
                    max: max,
                    theta: thetaMax,
                    phi: phiMax,
                    x: x,
                    y: y,
                    z: z
                }
            }

            //launched on drawing loop
            var do_physics=function() {
                //compute max attack vectors
                var ymax=get_max("top"),
                    ymin=get_max("bottom"),

                    xmax=get_max("right"),
                    xmin=get_max("left"),

                    zmin=get_max("front"),
                    zmax=get_max("back");

                

                var cy=SETTINGS.physics.c_y, cxz=SETTINGS.physics.c_xz, seuil=SETTINGS.physics.h_max;
                var dy=0, dx=0, dz=0;
                dy=-cy*(ymin.y+ymax.y);
                dx=cxz*(xmax.x+xmin.x);
                dz=cxz*(zmax.z+zmin.z);
                if (Math.abs(dx)<seuil) dx=0;
                if (Math.abs(dz)<seuil) dz=0;

                return [dx,dy,dz];
            }

            var v_y=0,
                G=SETTINGS.physics.gravity,
                jumped=false;

            var that={


                   draw: function(timestamp) {
                       dt=Math.min(SETTINGS.scene.dt_max, timestamp-old_timestamp);
                       old_timestamp=timestamp;
                       that.drawPhysics();

                       v_y+=G*dt;
                       var dy=v_y*dt;                       
                       spec.vue.moveCamera([0,dy,0]);

                        //DRAW PHYSIC CUBEMAP
                           SHADERS.set_physicShader();
                           SHADERS.set_rminmax(SETTINGS.physics.r_min, SETTINGS.physics.r_max);
                           GL.clearColor(0.0, 0.0, 1.0, 1.0);
                           GL.viewport(0, 0, physicsW, physicsW);                           
                           spec.vue.drawDistance();


                           //FRONT
                           GL.clear(GL.COLOR_BUFFER_BIT |GL.DEPTH_BUFFER_BIT);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[0]);
                           that.draw_objetsDistance();
                           GL.readPixels(0, 0, physicsW, physicsW, GL.RGBA, GL.UNSIGNED_BYTE, cubemaps_buffers.front);


                           //BOTTOM - GROUND
                           GL.clear(GL.COLOR_BUFFER_BIT |GL.DEPTH_BUFFER_BIT);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[2]);
                           that.draw_objetsDistance();
                           GL.readPixels(0, 0, physicsW, physicsW, GL.RGBA, GL.UNSIGNED_BYTE, cubemaps_buffers.bottom);

                           //TOP - SKY
                           GL.clear(GL.COLOR_BUFFER_BIT |GL.DEPTH_BUFFER_BIT);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[3]);
                           that.draw_objetsDistance();
                           GL.readPixels(0, 0, physicsW, physicsW, GL.RGBA, GL.UNSIGNED_BYTE, cubemaps_buffers.top);

                           //LEFT
                           GL.clear(GL.COLOR_BUFFER_BIT |GL.DEPTH_BUFFER_BIT);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[4]);
                           that.draw_objetsDistance();
                           GL.readPixels(0, 0, physicsW, physicsW, GL.RGBA, GL.UNSIGNED_BYTE, cubemaps_buffers.left);

                           //RIGHT
                           GL.clear(GL.COLOR_BUFFER_BIT |GL.DEPTH_BUFFER_BIT);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[5]);
                           that.draw_objetsDistance();
                           GL.readPixels(0, 0, physicsW, physicsW, GL.RGBA, GL.UNSIGNED_BYTE, cubemaps_buffers.right);

                           //BACK
                           GL.clear(GL.COLOR_BUFFER_BIT |GL.DEPTH_BUFFER_BIT);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[1]);
                           that.draw_objetsDistance();
                           GL.readPixels(0, 0, physicsW, physicsW, GL.RGBA, GL.UNSIGNED_BYTE, cubemaps_buffers.back);

                           var move=do_physics();
                           if (move[1]<0) {
                               //console.log(move[1]);
                               v_y=0;
                               jumped=false;
                           }
                           spec.vue.moveCamera(move);
                       

                       //drawing scene
                       SHADERS.set_defaultShader();

                       GL.viewport(0.0, 0.0, CV.width, CV.height);
                       GL.clearColor(0.0, 0.0, 0.0, 1.0);
                       GL.clear(GL.COLOR_BUFFER_BIT | GL.DEPTH_BUFFER_BIT);
                       spec.vue.draw();
                       lumieres.map(drawObjet);
                       that.draw_objets();


                       //physics
                       //GL.clear(GL.DEPTH_BUFFER_BIT);

                       if (debugDistance) {
                           SHADERS.set_physicShader();
                           GL.clearColor(0.0, 0.0, 1.0, 1.0);
                           GL.clear(GL.DEPTH_BUFFER_BIT);
                           spec.vue.drawDistance();

                           //FRONT
                           GL.viewport(1*physicsW, 1*physicsW, physicsW, physicsW);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[0]);
                           that.draw_objetsDistance();

                           //BOTTOM - GROUND
                           GL.viewport(1*physicsW, 0*physicsW, physicsW, physicsW);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[2]);
                           that.draw_objetsDistance();

                           //TOP - SKY
                           GL.viewport(1*physicsW, 2*physicsW, physicsW, physicsW);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[3]);
                           that.draw_objetsDistance();

                           //LEFT
                           GL.viewport(0*physicsW, 1*physicsW, physicsW, physicsW);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[4]);
                           that.draw_objetsDistance();

                           //RIGHT
                           GL.viewport(2*physicsW, 1*physicsW, physicsW, physicsW);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[5]);
                           that.draw_objetsDistance();

                           //BACK
                           GL.viewport(3*physicsW, 1*physicsW, physicsW, physicsW);
                           SHADERS.set_matriceProjection_distance(proj_matrix_cubeMap[1]);
                           that.draw_objetsDistance();

                       }
                       

                       GL.flush();

                       window.requestAnimationFrame(SCENE.draw);
                   },

                   jump: function() {
                       if (jumped) return false;
                       v_y=-SETTINGS.physics.jump_v0;
                       jumped=true;
                       return true;
                   },

                   draw_vue: function() {
                       spec.vue.draw();
                   },

                   draw_objets: function() {                       
                       objets.map(drawObjet);
                       //GL.enable(GL.BLEND);
                       //objets_alpha.map(drawObjet);
                       //GL.disable(GL.BLEND);
                   },

                   draw_objetsDistance: function() {
                       objets.map(drawObjetDistance);
                   },

                   drawPhysics: function() {
                       if (spec.navigation) spec.navigation.drawPhysics(dt);
                       spec.vue.drawPhysics(dt);
                       objets.map(drawObjetPhysics);
                       objets_alpha.map(drawObjetPhysics);
                   },

                   add_lumiere: function(lumiere) {
                       lumieres.push(lumiere);
                   },

                   add_objet: function(objet) {
                       objets.push(objet);
                   },

                   add_objetAlpha: function(objet) {
                       objets_alpha.push(objet);
                   },

                   set_vue: function(v) {
                       spec.vue=v;
                   }
            };
            SCENE=that;
            return that;
        }
    };
})();


