/*
 * spec.camera : position de la caméra. defaut : [0,0,0]
 * spec.theta : angle horizontal en radians. defaut : 0
 * spec.phi : angle vertical en radians. defaut : 0
 * spec.angle : angle de vue en degrés. Utilisé pour la matrice de projection. defaut : 45
 * spec.zMin : zMin de la matrice de projection. defaut : 1
 * spec. zMax : zMax de la matrice de projection. defaut : 10
 * spec.a : rapport L/H du canvas. defaut : CV.width/CV.height
 * spec.ortho : vue orthogonale. défaut : false
 * spec.largeur: largeur de la vue si vue orthogonale
 */
var Vue=(function() {
    return {
        instance: function(spec) {
            spec.camera=spec.camera || [0,0,0];
            spec.theta=spec.theta || 0;
            spec.phi=spec.phi ||0;
            spec.angle=spec.angle || 45;
            spec.zMin=spec.zMin || 1;
            spec.zMax=spec.zMax || 10;
            spec.a=spec.a || CV.width/CV.height;
            spec.ortho=spec.ortho || false;
            spec.largeur=spec.largeur || 5;

            var matrice_vue=lib_matrix4.get_I4();
            var matrice_projection;
            
            var matrice_vueDistance=lib_matrix4.get_I4();

            var calcule_matrice=function() {                

                lib_matrix4.set_I4(matrice_vue);                
                lib_matrix_rot4.rotateY(matrice_vue, spec.theta);
                lib_matrix_rot4.rotateX(matrice_vue, spec.phi);
                lib_matrix_mv.translateRot(matrice_vue, spec.camera);

                //console.log(spec.camera, spec.theta, spec.phi);
                lib_matrix_mv.setPos(matrice_vueDistance, spec.camera);
            }

            var calcule_matrice_projection=function(a) {
                matrice_projection=(spec.ortho)?lib_matrix_projection.getOrtho(spec.largeur, spec.a, spec.zMin, spec.zMax):lib_matrix_projection.get(spec.angle, a, spec.zMin, spec.zMax);
            }
            calcule_matrice_projection(spec.a);

            var that={
                draw: function() {
                    SHADERS.set_matriceVue(matrice_vue);
                    SHADERS.set_matriceProjection(matrice_projection);
                    //SHADERS.set_matriceProjection(lib_matrix_projection.cubeMatrices(1,500)[0]);
                    SHADERS.set_camera(spec.camera);
                },

                drawDistance: function() {                    
                    SHADERS.set_matriceVue_distance(matrice_vueDistance);
                },
                
                drawPhysics: function() {

                },

                get_matriceModeleVue: function() { //retourne la matrice modèle-vue                    
                    return lib_matrix4.multNew(matrice_vue, matrice_projection);
                },

                plein_ecran: function() {
                    var dimensionne_canvas=function(event) {
                        CV.width=window.innerWidth;
                        CV.height=window.innerHeight;
                        calcule_matrice_projection(CV.width/CV.height);
                    }
                    window.addEventListener("resize", dimensionne_canvas, true);
                    dimensionne_canvas();
                },

                moveAngles: function(dTheta, dPhi) {
                    spec.theta+=dTheta;
                    spec.phi+=dPhi;
                    calcule_matrice();
                },

                translateXVue: function(dX) {
                    spec.camera[0]+=dX*Math.cos(spec.theta);
                    spec.camera[2]+=dX*Math.sin(spec.theta);
                    calcule_matrice();
                },

                translateZVue: function(dY) {
                    spec.camera[0]+=dY*Math.sin(spec.theta);
                    spec.camera[2]-=dY*Math.cos(spec.theta);
                    calcule_matrice();
                },

                moveCamera: function(v) {
                    spec.camera[0]+=v[0],
                    spec.camera[1]+=v[1],
                    spec.camera[2]+=v[2];
                    calcule_matrice();
                }
            }
            calcule_matrice();            
            return that;
        }
    };
})();

